// =================================================================== //
// Time-stamp: <29 Mar 01 22:13:43 Jean-Guillaume.Dumas@imag.fr> 
// =================================================================== //
#ifndef __GIV_ALARM_H__
#define __GIV_ALARM_H__


#include <iostream.h>
#include <stdlib.h>

#include <setjmp.h> 
#include <signal.h>
#include <sys/time.h>

struct GivAlarm {
    enum { 
        MSPSEC = 1000000  // microsecond per second
    };

    GivAlarm(long sec, long usec = 0) {
        _rebours.it_value.tv_sec = sec;
        _rebours.it_value.tv_usec = usec;
    }
    GivAlarm(double time) {
        _rebours.it_value.tv_sec = (long)time;
        _rebours.it_value.tv_usec = (long)(MSPSEC*(time - _rebours.it_value.tv_sec));
   }

    virtual void operator() () = 0;

    bool run() {
        int  returned_from_longjump = sigsetjmp(env,1);
        if (returned_from_longjump != SIGALRM) {
            signal(SIGALRM, &signal_handler);
            signal(SIGPROF, &signal_handler);
            setitimer(ITIMER_REAL, &_rebours, NULL);
            operator() (); // Real Job
            return true;
        } 
        return false;
    }   

protected:
    static void signal_handler(int sig) {
       switch (sig)     {
            case SIGALRM:  /* process for alarm */
                siglongjmp(env,sig);
                    /* break never reached */
            default:       exit(sig);
        }
    }
    
    itimerval _rebours;
    static sigjmp_buf env;
};

#endif

